unit UnitBitmapRgn;

{-----------------------------------------------------}
{   This code was written by:                         }
{                              Carlos Prez Sancho    }
{                              cps@5alu.ua.es         }
{                                                     }
{   This component is FREEWARE, and it's distributed  }
{   A IS, WITHOUT ANY WARRANTIES. I'm not responsible }
{   for any damage caused by using TBitmapRgn         }
{-----------------------------------------------------}


{-----------------------------------------------------}
{                                                     }
{  READ THIS!!                                        }                                                     
{                                                     }
{  TBitmapRgn creates an irregular region from a      }
{   mask bitmap and applies it to its parent Form.    }
{                                                     }
{  HOW TO USE:                                        }
{  It's very easy. First of all install the package.  }
{  After you installed the package, add the component }
{  to your form and select a bitmap and a transparent }
{  color that will be the invisible area in your form }
{                                                     }
{  Once you've done this, just invoke the procedure   }
{  REGIONIZE in your code whenever you want, but      }
{  ONLY ONE TIME (see considerations below).          }
{                                                     }
{  CONSIDERATIONS:                                    }
{  1. Make sense that the mask bitmap can contain a   }
{     very complex area. In that case the region      }
{     won't be created and nothing will happen.       }
{                                                     }
{  2. If the region is created, but the region is too }
{     complex, the program will be very slow (I can't }
{     improve it. If you do it, please notify me)     }
{                                                     }
{  3. Once the region is created, the mask bitmap is  }
{     freed. If you want to apply te mask again       }
{     you'll have to reload the bitmap, but I can't   }
{     assure the performance of the program. If you   }
{     want to preserve the bitmap, just modify the    }
{     code in procedure Regionize.                    }
{                                                     }
{  4. I strongly recommend to use monochrome bitmaps  }
{     for the mask. Anyway, why would you need more?  }
{                                                     }
{  5. I also recommend not to use this component when }
{     creating regular regions that can be created    }
{     with CreateRectRgn, etc... I know it's very     }
{     boring but it'll result in a better performance }
{                                                     }
{  6. This code was developed using Delphi 4.0        }
{     I don't know how it will work under other       }
{     versions.                                       }                                                                                                          
{                                                     }
{-----------------------------------------------------}



interface

uses Windows, Graphics, Forms, Classes, Controls, Dialogs, SysUtils;

type
  TBitmapRgn = class(TComponent)
  private
    FMask: TBitmap;
    FColor: TColor;
    FControl: TWinControl;
    function CreateRegion: HRGN;
    procedure SetBitmap(value: TBitmap);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Regionize;
  published
    property Mask: TBitmap read FMask write SetBitmap;
    property TransparentColor: TColor read FColor write FColor;
  end;

procedure Register;

implementation

procedure Register;
begin
     RegisterComponents('Free', [TBitmapRgn]);
end;

constructor TBitmapRgn.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     if AOwner is TWinControl then
       FControl := TWinControl(AOwner);
     FMask := TBitmap.Create;
end;

destructor TBitmapRgn.Destroy;
begin
     if FMask<>nil then FMask.Free;
     FControl := nil;
     inherited Destroy;
end;

procedure TBitmapRgn.SetBitmap(value: TBitmap);
begin
     if FMask<>nil then FMask.Free;
     FMask := TBitmap.Create;
     FMask.Assign(value);
end;

{-----------------------------------------------------------------}
{ This is the heart of the component. This function makes the     }
{   region using the bitmap. It scans the bitmap and makes        }
{   a path in the window for every horizontal line in the bitmap. }
{   When finished, it translates the path into a region           }
{-----------------------------------------------------------------}

function TBitmapRgn.CreateRegion: HRGN;
var
   dc, dc_c: HDC;
   rgn: HRGN;
   x, y: integer;
   coord: TPoint;
   line: boolean;
   color: TColor;
begin
     dc := GetWindowDC(FControl.Handle);
     dc_c := CreateCompatibleDC(dc);
     SelectObject(dc_c, FMask.Handle);
     BeginPath(dc);
     for x:=0 to FMask.Width-1 do
     begin
          line := false;
          for y:=0 to FMask.Height-1 do
          begin
               color := GetPixel(dc_c, x, y);
               if not (color = FColor) then
               begin
                    if not line then
                    begin
                         line := true;
                         coord.x := x;
                         coord.y := y;
                    end;
               end;
               if (color = FColor) or (y=FMask.Height-1) then
               begin
                    if line then
                    begin
                         line := false;
                         MoveToEx(dc, coord.x, coord.y, nil);
                         LineTo(dc, coord.x, y);
                         LineTo(dc, coord.x + 1, y);
                         LineTo(dc, coord.x + 1, coord.y);
                         CloseFigure(dc);
                    end;
               end;
          end;
     end;
     EndPath(dc);
     rgn := PathToRegion(dc);
     ReleaseDC(FControl.Handle, dc);
     Result := rgn;
end;

procedure TBitmapRgn.Regionize;
var
   rgn: HRGN;
begin
     if (FMask<>nil) and (Owner<>nil) and (Owner is TWinControl) then begin
       rgn := CreateRegion;
       if rgn<>0 then begin
         SetWindowRgn(FControl.Handle, rgn, true);
         FControl.Width := FMask.Width;
         FControl.Height := FMask.Height;
       end;
       FMask.Free;
       FMask := TBitmap.Create;
     end;
end;

end.
